#!/usr/bin/env python
# coding: utf-8

# In[ ]:


class Clustering_hierarchical:
    """ Clustering is performed on the dissimilarity matrix chosen.
        Similarity in time: Uses the scaled data set and the calculation of the euclidean distance matrix
            1) Initilise the dataframes_scaled
            2) Call the distance matrix function 'distance_matrix_euc' with the corresponding weigthing vector
            3) Cluster and plot the dendrogram with 'plot_dendrogram' function
            4) Cut the dendrogram and plot the clusters with 'print_clusters' function
            
        Similarity in shape: Uses the normalised data set and the calculation of the DTW distance matrix.
            1) Initilise the dataframes_norm
            2) Call the distance matrix function 'distance_matrix_dtw' with the corresponding weigthing vector
            3) Same as similarity in time
            4) Same as similarity in time
            """
    def __init__(self, dataframes_norm = None, dataframes_scaled = None):
        self.dataframes_norm = dataframes_norm
        self.dataframes_scaled = dataframes_scaled
        self.dist_cond = None
        self.dist_cond_original = None
        
        
        
    def distance_matrix_euc(self, alphas):
        """ Alphas is the weighting vector of length equal to the number of dimension in the data set """
        # Initializing an empty condensed distance matrix
        condensed_matrix_size = 0
        for i in range(1,self.dataframes_scaled[0].shape[1]):
            condensed_matrix_size += i
        dist_cond = np.zeros([condensed_matrix_size,])
        dist_cond_original = []

        # Calculating the distance
        start = timeit.default_timer()
        for idx, df_s in enumerate(self.dataframes_scaled):
            dist_cond_original.append(ssd.pdist(df_s.T.values, metric='euclidean'))
            if alphas[idx] != 0:
                dist_cond = np.add(dist_cond, alphas[idx]*dist_cond_original[idx])

        end = timeit.default_timer()
        print("Time: ",end-start)
        print("Shape of dist: ", dist_cond.shape)
        
        # Mean condenced distance vector
        sum_alphas = sum(alphas)
        dist_cond /= sum_alphas
        
        self.dist_cond = dist_cond
        self.dist_cond_original = dist_cond_original
    

    
    def distance_matrix_dtw(self, alphas, window_size):
        """ Alphas is the weighting vector of length equal to the number of dimension in the data set """
        num_cores = multiprocessing.cpu_count()

        start = timeit.default_timer()
        temp = Parallel(n_jobs=num_cores)( delayed(dtw.distance_matrix)(df.T.values,
                use_c=True, show_progress=True, window=window_size) for df in self.dataframes_norm )
        end = timeit.default_timer()
        print("Time in seconds for calculating the dtw distance matrix: ",end-start)
    

        # Constructing the original untampered distance matrix; not including where alphas is zero.
        count_nonzero = np.count_nonzero(alphas)
        iterations = [1 for v in range(count_nonzero - 1)] 
        dist_squared_original = temp[0]
        for i in iterations:
            dist_squared_original = np.add(dist_squared_original, temp[i])
        print(dist_squared_original.shape)

        # Transfoming to condensed distance matrix (n,)
        listing = []
        for i in range(dist_squared_original.shape[0]):
            for j in range(i+1,dist_squared_original.shape[1]):
                listing.append(dist_squared_original[i,j])

        dist_cond_original = np.asarray(listing)
        print(dist_cond_original.shape)    


        # Combining the dissimilarity matrices: Used in the cluster analysis
        dist_squared = alphas[0]*temp[0]

        for i in range(1,len(temp)):
            if alphas[i] != 0:
                dist_squared = np.add(dist_squared, alphas[i]*temp[i])
        print(dist_squared.shape)    

        # Transfoming to condensed distance matrix (n,)
        listing = []
        for i in range(dist_squared.shape[0]):
            for j in range(i+1,dist_squared.shape[1]):
                listing.append(dist_squared[i,j])

        dist_cond = np.asarray(listing)
        dist_cond.shape

        # Mean condenced distance vector
        sum_alphas = sum(alphas)
        dist_cond /= sum_alphas
        dist_cond_original /= count_nonzero

        self.dist_cond = dist_cond
        self.dist_cond_original = dist_cond_original
    
    
    
    # Helper function for plotting the dendrogram
    def fancy_dendrogram(self, *args, **kwargs):
        max_d = kwargs.pop('max_d', None)
        if max_d and 'color_threshold' not in kwargs:
            kwargs['color_threshold'] = max_d
        annotate_above = kwargs.pop('annotate_above', 0)

        ddata = dendrogram(*args, **kwargs)

        if not kwargs.get('no_plot', False):
            plt.title(r'Hierarchical Clustering Dendrogram - linkage: '+method+', metric: Euclidean', fontsize=20)
            plt.xlabel(r'sample index', fontsize=18)
            plt.ylabel(r'distance', fontsize=18)
            plt.yticks(fontsize=15)
            for i, d, c in zip(ddata['icoord'], ddata['dcoord'], ddata['color_list']):
                x = 0.5 * sum(i[1:3])
                y = d[1]
                if y > annotate_above:
                    plt.plot(x, y, 'o', c=c)
                    #plt.annotate("%.3g" % y, (x, y), xytext=(0, -5),
                                 #textcoords='offset points',
                                 #va='top', ha='center')
            if max_d:
                plt.axhline(y=max_d, linestyle='--', c='k')
        return ddata
    
    
    # Doing hierarchical clustering and plotting the dendrogram
    def plot_dendrogram(self, method, metric, cut_distance, dataframes,  path_pictures, file_ending):
        """ This method performes hierarchical clustering by using the hierarchical clustering 
            algorithm of SciPy.
            method: The linkage criteria of the hierarchical clustering algorithm. See SciPy doc.
            metric: The internal distance metric. See SciPy doc.
            cut_distance: Plotting a horizontal line which indicates the cut of the dendrogram. 
                          If no cut is wanted increase the number above the larges plotted distance 
                          value in the plot
            dataframes: Only used to get the columns of the dataframes which will be the sample index
                        of the dendrogram.
            path_pictrure: Path where the dendrogram will be saved.
            file_ending: file ending of the saved dendrogram"""
        Z = hac.linkage(self.dist_cond, method=method, metric=metric,optimal_ordering =True)
        # Plot dendogram
        fig = plt.figure(figsize=(20, 10))
        self.fancy_dendrogram(
            Z,
            leaf_rotation=0.,  # rotates the x axis labels
            leaf_font_size=15.,  # font size for the x axis labels
            labels = dataframes[0].columns,
            color_threshold = cut_distance,
            max_d=cut_distance,  # plot a horizontal cut-off line
        )


        fig.savefig(path_pictures+'dendrogram_'+method+'_'+metric+file_ending+'.eps', bbox_inches='tight',
                    pad_inches=0)
        plt.show()
    
    
    
    def print_clusters(self, dataframes, method, metric, k, count_nonzero, path_pictures, save_plot=False):
        """ This method performes hierarchical clustering by using the hierarchical clustering algorithm of 
            SciPy and printing the different clusters.   
            dataframes: For plotting of the time series assigned to each cluster
            method: The linkage criteria of the hierarchical clustering algorithm. See SciPy doc.
            metric: The internal distance metric. See SciPy doc.
            k: number of clusters.
            count_nonzero: Number of nonzero elements in the weigthing vector alpha.
            path_pictrure: Path where the dendrogram will be saved."""
        style_code = ['-', '-', '-', '-', '-', '-', '-', '-', '-', '-',
                      '--', '--', '--', '--', '--', '--', '--', '--', '--', '--']
        style_color = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', 
                       '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf',
                       '#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', 
                       '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']
        # k Number of clusters I'd like to extract
        Z = hac.linkage(self.dist_cond, method=method, metric=metric, optimal_ordering=True)
        results = fcluster(Z, k, criterion='maxclust')

        # check the results
        s = pd.Series(results)
        clusters = s.unique()
        clusters.sort()
        print(clusters)
        #fig, axarr = plt.subplots(num_cuts, length_dataframe,figsize=(20,15))
        fig, axarr = plt.subplots(count_nonzero, num_cuts, figsize=(20,15))
        if count_nonzero > 1:
            for idx, c in enumerate(clusters):
                cluster_indeces = s[s==c].index
                print("Cluster %d number of entries %d" % (c, len(cluster_indeces)))
                for num, df in enumerate(dataframes):
                    custom_cycle = (cycler(color=style_color) + 
                                    cycler(linestyle=style_code))
                    
                    axarr[num,idx].set_prop_cycle(custom_cycle)
                    axarr[num,idx].plot(df.index, df.iloc[:,cluster_indeces])
                    plt.gcf().autofmt_xdate()


                axarr[0,idx].legend(dataframes[0].columns[cluster_indeces], loc='upper right', ncol=4, 
                                    fontsize=8, framealpha = 0.7)
                axarr[0, idx].set_title("Cluster "+str(idx+1), fontsize=20)
                # axarr[0, idx].set_ylim((-50,2600)) # Used for power time series
                # axarr[1, idx].set_ylim((5,70))     # Generator speed
                # axarr[3, idx].set_ylim((0,27))     # Wind speed
                #axarr[5, idx].set_ylim((-200,200))  # Position deviation or wind direction
            for num, df in enumerate(dataframes):
                axarr[num,0].set_ylabel(df_names[num], fontsize=15)
            
        elif count_nonzero == 1:
            for idx, c in enumerate(clusters):
                cluster_indeces = s[s==c].index
                print("Cluster %d number of entries %d" % (c, len(cluster_indeces)))
                df = dataframes[0]
                custom_cycle = (cycler(color=style_color) + 
                                    cycler(linestyle=style_code))
                axarr[idx].set_prop_cycle(custom_cycle)
                axarr[idx].plot(df.index, df.iloc[:,cluster_indeces])
                plt.gcf().autofmt_xdate()
                
                axarr[idx].legend(df.columns[cluster_indeces], loc='upper right', ncol=4, fontsize=8, 
                                  framealpha = 0.7)
                axarr[idx].set_title("Cluster "+str(idx+1), fontsize=20)
                # axarr[idx].set_ylim((-50,2600)) # Used for power time series
        else:
            print("Do nothing")
        
        if save_plot:
            plt.savefig(path_pictures+'subplots_'+method+'_numCut'+str(k)+'_dim4.pdf', bbox_inches='tight',
                        pad_inches=0)
             

