#!/usr/bin/env python
# coding: utf-8

# ## K-means 
# Implementation from scipy library
# 
# https://docs.scipy.org/doc/scipy/reference/cluster.vq.html#module-scipy.cluster.vq

# In[ ]:


### K-means from scipy.cluster.vq.kmeans

from scipy.cluster.vq import vq, kmeans, kmeans2, whiten
import timeit

num_clusters = 4
num_iterations = 5000

# Codebook: A k by N array of k centroids, 
# Distrotion: The mean (non-squared) Euclidean distance between the observations passed and the centroids 
#             generated. 
# codebook, distortion = kmeans(timeSeries, num_clusters, iter=500)
start = timeit.default_timer()
centroid, _ = kmeans(dataframes.dataframes_scaled[0].T, num_clusters, iter= num_iterations, thresh=1e-08,
                     check_finite=False)
end = timeit.default_timer()

print("Time: ",(end-start)/num_iterations)

# Code: A length M array holding the code book index for each observation.
# dist: The distortion (distance) between the observation and its nearest code.
label, dist = vq(timeSeries, centroid)

print(label, dist)

df_centroid = pd.DataFrame(centroid.transpose())
df_centroid.set_index(time_axis.values, inplace=True)


# In[ ]:


# Getting the cluster centers and plotting them

import matplotlib.pyplot as plt
df_centroid.plot(figsize=(18,10), style=style_code, fontsize=15)
plt.title(r'Gearbox oil temperature', fontsize=20)
plt.ylabel(r'Temperature [C]', fontsize=18)

# Attaching lengeds
legends = []
for i in range(centroid.shape[0]):
    legends.append('Cluster '+str(i+1))
plt.legend(legends, fontsize=13)
plt.savefig(path_pictures+'Cluster_prototypes_numClusters_'+str(num_clusters)+'v2.eps', 
            bbox_inches='tight', pad_inches=0)
plt.show()
#bottom, top = [0,1]
#plt.ylim(bottom, top)

# Cluster assignment
print("Cluster assignment: ",label)


# In[ ]:


# Plotting the cluster center along with the time series assigned to that cluster

for idx, cluster in enumerate(centroid):
    # Transforming to panda frames
    labels = pd.Series(label)
    cluster_plot = pd.DataFrame(cluster)
    cluster_plot.set_index(time_axis.values, inplace=True)
    cluster_plot.columns = ['Cluster '+str(idx+1)]
    
    # Assigning time-series to correct cluster
    cluster_indeces = labels[labels==idx].index
    print("Cluster %d number of entries %d" % (idx+1, len(cluster_indeces)))

    # Plotting the results
    title = r"Cluster "+str(idx+1)+" - Number of time-series: "+str(len(cluster_indeces))
    ax = cluster_plot.plot(figsize=(18,8), title = title, style=style_code, fontsize = 20)
    timeSeries.T.iloc[:,cluster_indeces].plot(ax=ax, style=style_code, alpha=0.3, fontsize = 20)
    ax.legend(loc='upper right', fontsize=20)
    ax.set_ylim(bottom, top)
    fig = ax.get_figure()

