#!/usr/bin/env python
# coding: utf-8

# # Comparison between different DTW implementations

# In[1]:


import timeit
import numpy as np
import matplotlib.pyplot as plt
from dtaidistance import dtw
from dtaidistance import dtw_visualisation as dtwvis
import numpy as np


# In[2]:


idx = np.linspace(0, 10, 5000)
x = 1.4*np.sin(idx)
y = np.cos(idx)

width = len(x)//10
print("Width:", width)
plt.figure(figsize=(15,10))
plt.plot(idx,x,idx,y)


# # DTW unconstrained

# In[3]:


from cdtw import pydtw

start = timeit.default_timer()
d = pydtw.dtw(x,y,pydtw.Settings(dist = 'euclid', step  = 'dp2', window = 'nowindow', 
                                 compute_path = False))
end = timeit.default_timer()
print("Time pydtw from cdtw: ",end-start)
print("Distance pydtw from cdtw: ", d.get_dist())


# In[4]:


from scipy.spatial.distance import euclidean
from fastdtw import fastdtw

start = timeit.default_timer()
distance, path = fastdtw(x, y, dist=euclidean)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)


# In[5]:


from tslearn.metrics import dtw_path

start = timeit.default_timer()
path, distance = dtw_path(x, y) # support
end = timeit.default_timer()
# 
print("Time: ",end-start)
print("Distance: ", distance)


# In[6]:


from tslearn.metrics import cdist_dtw

start = timeit.default_timer()
path, distance = cdist_dtw([x,y])
end = timeit.default_timer()
# 
print("Time: ",end-start)
print("Distance: ", distance)


# In[7]:


from dtaidistance import dtw

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y])
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_c = True)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_nogil  = True)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_nogil  = True, parallel = True)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_c  = True, parallel = True)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix_fast([x,y])
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)


# # DTW contrained, Sakoe-band

# In[8]:


from tslearn.metrics import dtw_path

start = timeit.default_timer()
path, distance = dtw_path(x, y, global_constraint='sakoe_chiba', sakoe_chiba_radius=width) # support
end = timeit.default_timer()
# 
print("Time: ",end-start)
print("Distance: ", distance)


# In[9]:


from tslearn.metrics import cdist_dtw

start = timeit.default_timer()
path, distance = cdist_dtw([x,y], global_constraint='sakoe_chiba', sakoe_chiba_radius=width)
end = timeit.default_timer()
# 
print("Time: ",end-start)
print("Distance: ", distance)


# In[3]:


from dtaidistance import dtw

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], window=width)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_c = True, window=width)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_nogil  = True, window=width)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_nogil  = True, parallel = True, window=width)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)

start = timeit.default_timer()
distance = dtw.distance_matrix([x,y], use_c  = True, parallel = True, window=width)
end = timeit.default_timer()

print("Time: ",end-start)
print("Distance: ", distance)


# # IMPLEMENTATION DOES NOT WORK!
# start = timeit.default_timer()
# distance = dtw.distance_matrix_fast([x,y], window=width)
# end = timeit.default_timer()

# # IMPLEMENTATION DOES NOT WORK!
# start = timeit.default_timer()
# distance = dtw.distance_fast(x,y, window=width)
# end = timeit.default_timer()

